/*
 * Copyright 2000 by Jean-Claude Christophe <jch@oleane.net>
 *                                          <jch@bsdfr.org>
 *
 * Home page:  http://www.bsdfr.org/projects/tac_bsd/
 *
 * draft2.h  This file defines all constants, variables and structures
 *           need by the tac_+ protocol.
 */

/* 
 * Format of packet header and body header are at the bottom of this
 * documents.
 */

#ifndef _DRAFT2_H_
#define _DRAFT2_H_

#include <sys/types.h> /* u_char and u_short */

#define HEADER_SIZE                12   /* Header always contains 12 bytes */
#define AUTHEN_START_SIZE          8
#define AUTHEN_CONT_SIZE           5

/* all following defines come from the draft2 of TACACS+ */

#define TAC_PLUS_ENCRYPTED_FLAG    0x00  /* Packet is encrypted */
#define TAC_PLUS_UNENCRYPTED_FLAG  0x01  /* Packet is not encrypted */

/* --- header */
/* major_version */
#define TAC_PLUS_MAJOR_VER         0xc0  /* 192 */

/* minor version */
#define TAC_PLUS_MINOR_VER_DEFAULT 0x0
#define TAC_PLUS_MINOR_VER_ONE     0x01

/* type */
#define TAC_PLUS_AUTHEN            0x01
#define TAC_PLUS_AUTHOR            0x02
#define TAC_PLUS_ACCT              0x03

/* --- body */
/* action */
#define TAC_PLUS_AUTHEN_LOGIN      0x01
#define TAC_PLUS_AUTHEN_CHPASS     0x02
#define TAC_PLUS_AUTHEN_SENDPASS   0x03
#define TAC_PLUS_AUTHEN_SENDAUTH   0x04

/* priv_lvl */
#define TAC_PLUS_PRIV_LVL_MAX      0x0f
#define TAC_PLUS_PRIV_LVL_ROOT     0x0f
#define TAC_PLUS_PRIV_LVL_USER     0x01
#define TAC_PLUS_PRIV_LVL_MIN      0x00

/* authen_type */
#define TAC_PLUS_AUTHEN_TYPE_ASCII 0x01
#define TAC_PLUS_AUTHEN_TYPE_PAP   0x02
#define TAC_PLUS_AUTHEN_TYPE_CHAP  0x03
#define TAC_PLUS_AUTHEN_TYPE_ARAP  0x04
#define TAC_PLUS_AUTHEN_TYPE_MSCHAP 0x05

/* service */
#define TAC_PLUS_AUTHEN_SVC_NONE   0x00
#define TAC_PLUS_AUTHEN_SVC_LOGIN  0x01
#define TAC_PLUS_AUTHEN_SVC_ENABLE 0x02
#define TAC_PLUS_AUTHEN_SVC_PPP    0x03
#define TAC_PLUS_AUTHEN_SVC_ARAP   0x04
#define TAC_PLUS_AUTHEN_SVC_PT     0x05
#define TAC_PLUS_AUTHEN_SVC_RCMD   0x06
#define TAC_PLUS_AUTHEN_SVC_X25    0x07
#define TAC_PLUS_AUTHEN_SVC_NASI   0x08
#define TAC_PLUS_AUTHEN_SVC_FWPROXY 0x09

/* --- authentication reply */
/* status */
#define TAC_PLUS_AUTHEN_STATUS_PASS    0x01
#define TAC_PLUS_AUTHEN_STATUS_FAIL    0x02
#define TAC_PLUS_AUTHEN_STATUS_GETDATA 0x03
#define TAC_PLUS_AUTHEN_STATUS_GETUSER 0x04
#define TAC_PLUS_AUTHEN_STATUS_GETPASS 0x05
#define TAC_PLUS_AUTHEN_STATUS_RESTART 0x06
#define TAC_PLUS_AUTHEN_STATUS_ERROR   0x07
#define TAC_PLUS_AUTHEN_STATUS_FOLLOW  0x21

/* struct of header */
typedef struct s_pkt_hdr {
    u_char version; /* Major version and Minor version */
    u_char type;    /* Type (Authentication, Authorization or Accounting) */
    u_char seq_no;  /* packet sequence number */
    u_char flags;   /* One byte who defines if we encrypt or not datas. */
    int session_id; /* Number of the session ID */
    int length;     /* length of datas after this header */
} t_pkt_hdr;

/* The authentication START structure */
typedef struct s_send_authen {
    u_char action;       /* Authentication action to be performed */
    u_char priv_lvl;     /* The privilege level */
    u_char authen_type;  /* The type of authentication that is being performed */
    u_char service;      /* This is the service that is requesting the authentication */
    u_char user_len;     /* Size of the username */
    u_char port_len;     /* Size of the ASCII name of the client port (ttyX for cisco) */
    u_char rem_addr_len; /* Size of an ASCII string that describes the user's remote location */
    u_char data_len;     /* rest of datas.. */
} t_send_authen;

/* The authentication reply structure */
typedef struct s_authen_reply {
    u_char status;
    u_char flags;
    u_short msg_len;
    u_short data_len;
    char msg_data[2048];
} t_authen_reply;

/* The authentication continue structure */
typedef struct s_cont_authen {
    u_short user_msg_len;
    u_short user_data_len;
    u_char flags;
} t_cont_authen;


#endif _DRAFT2_H_ /* _DRAFT2_H_ */

/*
  
  (All this informations are taken from draft2 of the TACACS+ definition)
  
  Packet Header:
  
  1 2 3 4 5 6 7 8  1 2 3 4 5 6 7 8  1 2 3 4 5 6 7 8  1 2 3 4 5 6 7 8
  
  +----------------+----------------+----------------+----------------+
  |major  | minor  |                |                |                |
  |version| version|      type      |     seq_no     |   flags        |
  +----------------+----------------+----------------+----------------+
  |                                                                   |
  |                            session_id                             |
  +----------------+----------------+----------------+----------------+
  |                                                                   |
  |                              length                               |
  +----------------+----------------+----------------+----------------+
  
  The authentication START packet body:
  
  1 2 3 4 5 6 7 8  1 2 3 4 5 6 7 8  1 2 3 4 5 6 7 8  1 2 3 4 5 6 7 8
  
  +----------------+----------------+----------------+----------------+
  |    action      |    priv_lvl    |  authen_type   |     service    |
  +----------------+----------------+----------------+----------------+
  |    user len    |    port len    |  rem_addr len  |    data len    |
  +----------------+----------------+----------------+----------------+
  |    user ...
  +----------------+----------------+----------------+----------------+
  |    port ...
  +----------------+----------------+----------------+----------------+
  |    rem_addr ...
  +----------------+----------------+----------------+----------------+
  |    data...
  +----------------+----------------+----------------+----------------+
  
*/
